/*
 * RARFile.java
 *
 * Created on September 30, 2001, 9:30 PM
 */

import java.io.*;
import java.util.*;

import util.*;

/**
 *
 * @author  jbanes
 * @version 
 */
public class RARFile 
{
    private File file;
    private Vector filelist = new Vector();
    
    static
    {
        System.loadLibrary("jrar");
    }
    
    public RARFile(File file) throws FileNotFoundException
    {
        this.file = file;
        
        if(!file.exists()) throw new FileNotFoundException("Could not find "+file.getPath());
    }

    public RARArchivedFile[] getArchivedFiles()
    {
        if(filelist.size() < 1) loadList(file.getPath());
     
        RARArchivedFile[] list = new RARArchivedFile[filelist.size()];
        
        for(int i=0; i<list.length; i++)
        {
            list[i] = (RARArchivedFile)filelist.elementAt(i);
        }
        
        return list;
    }
    
    private native void loadList(String filename);
    
    private native byte[] getData(String filename, String arcfile, String password);
    
    public InputStream extract(RARArchivedFile arc)
    {
        return extract(arc, "");
    }
    
    public InputStream extract(RARArchivedFile arc, String password)
    {
        return new BufferInputStream(getData(file.getPath(), arc.getName(), password));
    }
    
    private void addFile(String name, long packedSize, long unpackedSize, long timestamp, long attributes)
    {
        filelist.add(new RARArchivedFile(name, packedSize, unpackedSize, timestamp, attributes));
    }
    
    private static void print(String data, int length)
    {
        System.out.print(data);
        
        for(int i=0; i<length-data.length(); i++) System.out.print(' ');
    }
    
    public static void main(String[] args)
    {
        String filename = "c:\\Development\\UniquERAR\\rars\\best.rar";
        
        if(args.length > 0) filename = args[0];
        
        try
        {
            RARFile file = new RARFile(new File(filename));
            RARArchivedFile[] list = file.getArchivedFiles();
            
            InputStream in;
            OutputStream out;
            
            int data = 0;
            
            System.out.println("Printing "+list.length+" components. ");
            
            for(int i=0; i<list.length; i++)
            {
                print(list[i].getName(), 24);
                print(list[i].isDirectory()+"", 6);
                print(Long.toString(list[i].getPackedSize()), 10);
                print(Long.toString(list[i].getUnpackedSize()), 10);
                print(Long.toString(list[i].getTimestamp()), 12);
                print(Long.toBinaryString(list[i].getAttributes()), 16);
                System.out.println();
            }
            
            in = file.extract(list[0]);
            out = new FileOutputStream(list[0].getName());
            
            while((data = in.read()) >= 0)
            {
                out.write(data);
            }
            
            in.close();
            out.close();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        
        System.out.println("Done.");
    }
}
